@@ -1,10 +1,7 @@
 #pragma once
-
-// dont hook libc calls
-#define NO_XSLOCKS
-#define PERLIO_NOT_STDIO 0
-
-#define PERL_NO_GET_CONTEXT
+#define NO_XSLOCKS          // dont hook libc calls
+#define PERLIO_NOT_STDIO 0  // dont hook IO
+#define PERL_NO_GET_CONTEXT // we want efficiency for threaded perls
 extern "C" {
 #  include "EXTERN.h"
 #  include "perl.h"
@@ -35,6 +32,23 @@ typedef IO OIO;
     hv_common((hv), NULL, HEK_KEY(hek), HEK_LEN(hek), HEK_UTF8(hek), (flags)|HV_DELETE, NULL, HEK_HASH(hek))
 #endif
 
+#define XS_HV_ITER(hv,code) {                                                       \
+    STRLEN hvmax = HvMAX(hv);                                                       \
+    HE** hvarr = HvARRAY(hv);                                                       \
+    if (HvUSEDKEYS(hv))                                                             \
+        for (STRLEN bucket_num = 0; bucket_num <= hvmax; ++bucket_num)              \
+            for (const HE* he = hvarr[bucket_num]; he; he = HeNEXT(he)) { code }    \
+}
+#define XS_HV_ITER_NU(hv,code) XS_HV_ITER(hv,{if(!SvOK(HeVAL(he))) continue; code})
+
+#define XS_AV_ITER(av,code) {                                           \
+    SV** list = AvARRAY(av);                                            \
+    SSize_t fillp = AvFILLp(av);                                        \
+    for (SSize_t i = 0; i <= fillp; ++i) { SV* elem = *list++; code }   \
+}
+#define XS_AV_ITER_NE(av,code) XS_AV_ITER(av,{if(!elem) continue; code})
+#define XS_AV_ITER_NU(av,code) XS_AV_ITER(av,{if(!elem || !SvOK(elem)) continue; code})
+
 namespace xs {
 
 enum next_t {
